@@ -41,6 +41,7 @@ class Validator():
         ]
         # The following actually initialized in initialize() method
         self.id = None
+        self.spec_version = None
         self.digest_algorithm = None
         self.content_directory = None
         self.inventory_digest_files = None
@@ -54,6 +55,7 @@ class Validator():
         Must be called between attempts to validate objects.
         """
         self.id = None
+        self.spec_version = '1.0'  # default to latest published version
         self.digest_algorithm = 'sha512'
         self.content_directory = 'content'
         self.inventory_digest_files = {}  # index by version_dir, algorithms may differ
@@ -77,16 +79,34 @@ class Validator():
                 self.obj_fs = path
                 path = self.obj_fs.desc('')
         except fs.errors.CreateFailed:
-            self.log.error('E003c', path=path)
+            self.log.error('E003e', path=path)
             return False
-        # Object declaration
+        # Object declaration, set spec version number. If there are multiple declarations,
+        # look for the lastest object version then report any others as errors
         namastes = find_namastes(0, pyfs=self.obj_fs)
         if len(namastes) == 0:
-            self.log.error('E003a')
-        elif len(namastes) > 1:
-            self.log.error('E003b', files=len(namastes))
-        elif not namastes[0].content_ok(pyfs=self.obj_fs):
-            self.log.error('E007')
+            self.log.error('E003a', assumed_version=self.spec_version)
+        else:
+            spec_version = None
+            for namaste in namastes:
+                # Extract and check spec version number
+                this_file_version = None
+                for version in ('1.1', '1.0'):
+                    if namaste.filename == '0=ocfl_object_' + version:
+                        this_file_version = version
+                        break
+                if this_file_version is None:
+                    self.log.error('E006', filename=namaste.filename)
+                elif spec_version is None or this_file_version > spec_version:
+                    spec_version = this_file_version
+                    if not namaste.content_ok(pyfs=self.obj_fs):
+                        self.log.error('E007', filename=namaste.filename)
+            if spec_version is None:
+                self.log.error('E003c', assumed_version=self.spec_version)
+            else:
+                self.spec_version = spec_version
+                if len(namastes) > 1:
+                    self.log.error('E003b', files=len(namastes), using_version=self.spec_version)
         # Object root inventory file
         inv_file = 'inventory.json'
         if not self.obj_fs.exists(inv_file):
@@ -102,22 +122,27 @@ class Validator():
             self.digest_algorithm = inv_validator.digest_algorithm
             self.validate_inventory_digest(inv_file, self.digest_algorithm)
             # Object root
-            self.validate_object_root(all_versions)
+            self.validate_object_root(all_versions, already_checked=[namaste.filename for namaste in namastes])
             # Version inventory files
-            prior_manifest_digests = self.validate_version_inventories(all_versions)
+            (prior_manifest_digests, prior_fixity_digests) = self.validate_version_inventories(all_versions)
             if inventory_is_valid:
                 # Object content
-                self.validate_content(inventory, all_versions, prior_manifest_digests)
+                self.validate_content(inventory, all_versions, prior_manifest_digests, prior_fixity_digests)
         except ValidatorAbortException:
             pass
         return self.log.num_errors == 0
 
-    def validate_inventory(self, inv_file, where='root'):
+    def validate_inventory(self, inv_file, where='root', extract_spec_version=False):
         """Validate a given inventory file, record errors with self.log.error().
 
         Returns inventory object for use in later validation
         of object content. Does not look at anything else in the
         object itself.
+
+        where - used for reporting messages of where inventory is in object
+
+        extract_spec_version - if set True will attempt to take spec_version from the
+            inventory itself instead of using the spec_version provided
         """
         try:
             with self.obj_fs.openbin(inv_file, 'r') as fh:
@@ -126,8 +151,9 @@ class Validator():
             self.log.error('E033', where=where, explanation=str(e))
             raise ValidatorAbortException
         inv_validator = InventoryValidator(log=self.log, where=where,
-                                           lax_digests=self.lax_digests)
-        inv_validator.validate(inventory)
+                                           lax_digests=self.lax_digests,
+                                           spec_version=self.spec_version)
+        inv_validator.validate(inventory, extract_spec_version=extract_spec_version)
         return inventory, inv_validator
 
     def validate_inventory_digest(self, inv_file, digest_algorithm, where="root"):
@@ -159,24 +185,28 @@ class Validator():
         else:
             self.log.error("E058b", inv_digest_file=inv_digest_file)
 
-    def validate_object_root(self, version_dirs):
+    def validate_object_root(self, version_dirs, already_checked):
         """Validate object root.
 
         All expected_files must be present and no other files.
         All expected_dirs must be present and no other dirs.
         """
-        expected_files = ['0=ocfl_object_1.0', 'inventory.json',
+        expected_files = ['0=ocfl_object_' + self.spec_version, 'inventory.json',
                           'inventory.json.' + self.digest_algorithm]
         for entry in self.obj_fs.scandir(''):
             if entry.is_file:
-                if entry.name not in expected_files:
+                if entry.name not in expected_files and entry.name not in already_checked:
                     self.log.error('E001a', file=entry.name)
             elif entry.is_dir:
                 if entry.name in version_dirs:
                     pass
                 elif entry.name == 'extensions':
                     self.validate_extensions_dir()
+                elif re.match(r'''v\d+$''', entry.name):
+                    # Looks like a version directory so give more specific error
+                    self.log.error('E046b', dir=entry.name)
                 else:
+                    # Simply an unexpected directory
                     self.log.error('E001b', dir=entry.name)
             else:
                 self.log.error('E001c', entry=entry.name)
@@ -204,8 +234,9 @@ class Validator():
         so that we can also check them when validating the content.
         """
         prior_manifest_digests = {}  # file -> algorithm -> digest -> [versions]
+        prior_fixity_digests = {}  # file -> algorithm -> digest -> [versions]
         if len(version_dirs) == 0:
-            return prior_manifest_digests
+            return prior_manifest_digests, prior_fixity_digests
         last_version = version_dirs[-1]
         for version_dir in version_dirs:
             inv_file = fs.path.join(version_dir, 'inventory.json')
@@ -242,8 +273,9 @@ class Validator():
                     if len(not_seen) > 0:
                         self.log.error('E023b', where=version_dir, missing_filepaths=', '.join(sorted(not_seen)))
                     # Record all prior digests
-                    for digest in version_inventory['manifest']:
-                        for filepath in version_inventory['manifest'][digest]:
+                    for unnormalized_digest in version_inventory['manifest']:
+                        digest = normalized_digest(unnormalized_digest, digest_type=digest_algorithm)
+                        for filepath in version_inventory['manifest'][unnormalized_digest]:
                             if filepath not in prior_manifest_digests:
                                 prior_manifest_digests[filepath] = {}
                             if digest_algorithm not in prior_manifest_digests[filepath]:
@@ -254,9 +286,23 @@ class Validator():
                 # Is this inventory an appropriate prior version of the object root inventory?
                 if self.root_inv_validator is not None:
                     self.root_inv_validator.validate_as_prior_version(inv_validator)
-        return prior_manifest_digests
+                # Fixity blocks are independent in each version. Record all values and the versions
+                # they occur in for later checks against content
+                if 'fixity' in version_inventory:
+                    for digest_algorithm in version_inventory['fixity']:
+                        for unnormalized_digest in version_inventory['fixity'][digest_algorithm]:
+                            digest = normalized_digest(unnormalized_digest, digest_type=digest_algorithm)
+                            for filepath in version_inventory['fixity'][digest_algorithm][unnormalized_digest]:
+                                if filepath not in prior_fixity_digests:
+                                    prior_fixity_digests[filepath] = {}
+                                if digest_algorithm not in prior_fixity_digests[filepath]:
+                                    prior_fixity_digests[filepath][digest_algorithm] = {}
+                                if digest not in prior_fixity_digests[filepath][digest_algorithm]:
+                                    prior_fixity_digests[filepath][digest_algorithm][digest] = []
+                                prior_fixity_digests[filepath][digest_algorithm][digest].append(version_dir)
+        return prior_manifest_digests, prior_fixity_digests
 
-    def validate_content(self, inventory, version_dirs, prior_manifest_digests):
+    def validate_content(self, inventory, version_dirs, prior_manifest_digests, prior_fixity_digests):
         """Validate file presence and content against inventory.
 
         The root inventory in `inventory` is assumed to be valid and safe to use
@@ -288,7 +334,22 @@ class Validator():
                     else:
                         self.log.error("E015", where=version_dir, entry=entry)
             except (fs.errors.ResourceNotFound, fs.errors.DirectoryExpected):
-                self.log.error('E046', version_dir=version_dir)
+                self.log.error('E046a', version_dir=version_dir)
+        # Extract any digests in fixity and organize by filepath
+        fixity_digests = {}
+        if 'fixity' in inventory:
+            for digest_algorithm in inventory['fixity']:
+                for digest in inventory['fixity'][digest_algorithm]:
+                    for filepath in inventory['fixity'][digest_algorithm][digest]:
+                        if filepath in files_seen:
+                            if filepath not in fixity_digests:
+                                fixity_digests[filepath] = {}
+                            if digest_algorithm not in fixity_digests[filepath]:
+                                fixity_digests[filepath][digest_algorithm] = {}
+                            if digest not in fixity_digests[filepath][digest_algorithm]:
+                                fixity_digests[filepath][digest_algorithm][digest] = ['root']
+                        else:
+                            self.log.error('E093b', where='root', digest_algorithm=digest_algorithm, digest=digest, content_path=filepath)
         # Check all files in root manifest
         if 'manifest' in inventory:
             for digest in inventory['manifest']:
@@ -299,34 +360,45 @@ class Validator():
                         if self.check_digests:
                             content_digest = file_digest(filepath, digest_type=self.digest_algorithm, pyfs=self.obj_fs)
                             if content_digest != normalized_digest(digest, digest_type=self.digest_algorithm):
-                                self.log.error('E092a', where='root', digest=digest, content_path=filepath, content_digest=content_digest)
+                                self.log.error('E092a', where='root', digest_algorithm=self.digest_algorithm, digest=digest, content_path=filepath, content_digest=content_digest)
+                            known_digests = {self.digest_algorithm: content_digest}
+                            # Are there digest values in the fixity block?
+                            self.check_additional_digests(filepath, known_digests, fixity_digests, 'E093a')
                             # Are there other digests for this same file from other inventories?
-                            # If so then check those also
-                            if filepath in prior_manifest_digests:
-                                for digest_algorithm in prior_manifest_digests[filepath]:
-                                    for other_digest in prior_manifest_digests[filepath][digest_algorithm]:
-                                        content_digest = file_digest(filepath, digest_type=digest_algorithm, pyfs=self.obj_fs)
-                                        if content_digest != normalized_digest(other_digest, digest_type=digest_algorithm):
-                                            where = ','.join(prior_manifest_digests[filepath][digest_algorithm][other_digest])
-                                            self.log.error('E092a', where=where, digest=other_digest, content_path=filepath, content_digest=content_digest)
-                            # FIXME - Also other fixity blocks
+                            self.check_additional_digests(filepath, known_digests, prior_manifest_digests, 'E092a')
+                            self.check_additional_digests(filepath, known_digests, prior_fixity_digests, 'E093a')
                         files_seen.discard(filepath)
-        # Check any additional digests in root fixity block
-        if 'fixity' in inventory and self.check_digests:
-            for digest_algorithm in inventory['fixity']:
-                for digest in inventory['fixity'][digest_algorithm]:
-                    if digest != self.digest_algorithm:  # don't recheck things we check from manifest
-                        for filepath in inventory['fixity'][digest_algorithm][digest]:
-                            try:
-                                content_digest = file_digest(filepath, digest_type=digest_algorithm, pyfs=self.obj_fs)
-                                if content_digest != normalized_digest(digest, digest_type=digest_algorithm):
-                                    self.log.error('E093a', where='root', digest_algorithm=digest_algorithm, digest=digest, content_path=filepath, content_digest=content_digest)
-                            except fs.errors.ResourceNotFound:
-                                self.log.error('E093b', where='root', digest_algorithm=digest_algorithm, digest=digest, content_path=filepath)
         # Anything left in files_seen is not mentioned in the inventory
         if len(files_seen) > 0:
             self.log.error('E023a', where='root', extra_files=', '.join(sorted(files_seen)))
 
+    def check_additional_digests(self, filepath, known_digests, additional_digests, error_code):
+        """Check all the additional digests for filepath.
+
+        This method is intended to be used both for manifest digests in prior versions and
+        for fixity digests. The digests_seen dict is used to store any values calculated
+        so that we don't recalculate digests that might appear multiple times. It is added to
+        with any additional values calculated.
+
+        Parameters:
+            filepath - path of file in object (`v1/content/something` etc.)
+            known_digests - dict of algorithm->digest that we have calculated
+            additional_digests - dict: filepath -> algorithm -> digest -> [versions appears in]
+            error_code - error code to log on mismatch (E092a for manifest, E093a for fixity)
+        """
+        if filepath in additional_digests:
+            for digest_algorithm in additional_digests[filepath]:
+                if digest_algorithm in known_digests:
+                    # Don't recompute anything, just use it if we've seen it before
+                    content_digest = known_digests[digest_algorithm]
+                else:
+                    content_digest = file_digest(filepath, digest_type=digest_algorithm, pyfs=self.obj_fs)
+                    known_digests[digest_algorithm] = content_digest
+                for digest in additional_digests[filepath][digest_algorithm]:
+                    if content_digest != normalized_digest(digest, digest_type=digest_algorithm):
+                        where = ','.join(additional_digests[filepath][digest_algorithm][digest])
+                        self.log.error(error_code, where=where, digest_algorithm=digest_algorithm, digest=digest, content_path=filepath, content_digest=content_digest)
+
     def read_inventory_digest(self, inv_digest_file):
         """Read inventory digest from sidecar file.
 
